#include <iostream>
#include <fstream>
#include <vector>

#include <GL/glew.h>
#include "LisEtCompileLesShaders.h"

GLuint LisEtCompileLesShaders(const char * vertex_file_path,const char * fragment_file_path){

	// Create the shaders
	GLuint CleDuShaderDeSommet = glCreateShader(GL_VERTEX_SHADER);
	GLuint CleDuShaderDeFragment = glCreateShader(GL_FRAGMENT_SHADER);

	// LECTURE DU CODE SOURCE DU SHADER DE SOMMET
	std::string CodeSourceDuShaderDeSommet;
	{
		std::ifstream LectureDeFichier(vertex_file_path, std::ios::in);
		if(LectureDeFichier.is_open())
		{
			std::string Line = "";
			while(getline(LectureDeFichier, Line))
				CodeSourceDuShaderDeSommet += "\n" + Line;
			LectureDeFichier.close();
		}
		else
		{
			std::cout << "Impossible d'ouvrir " << vertex_file_path << ". etes vous dans le bon repertoire ?" << std::endl;
			return 0;
		}
	}

	// LECTURE DU CODE SOURCE DU SHADER DE FRAGMENT
	std::string CodeSourceDuShaderDeFragment;
	{
		std::ifstream LectureDeFichier(fragment_file_path, std::ios::in);
		if(LectureDeFichier.is_open())
		{
			std::string Line = "";
			while(getline(LectureDeFichier, Line))
				CodeSourceDuShaderDeFragment += "\n" + Line;
			LectureDeFichier.close();
		}
		else
		{
			std::cout << "Impossible d'ouvrir " << fragment_file_path << ". etes vous dans le bon repertoire ?" << std::endl;
			return 0;
		}
	}

	GLint ResultatTraitement = GL_FALSE;
	int InfoLogLongueur;

	// COMPILATION DU SHADER DE SOMMET
	std::cout << "Compilation du shader de sommet : " << vertex_file_path << std::endl;
	{
		char const * PointeurChaine = CodeSourceDuShaderDeSommet.c_str();
		glShaderSource(CleDuShaderDeSommet, 1, &PointeurChaine , NULL);
	}
	glCompileShader(CleDuShaderDeSommet);

	// VERIFICATION DE LA COMPILATION DU SHADER DE SOMMET
	glGetShaderiv(CleDuShaderDeSommet, GL_COMPILE_STATUS, &ResultatTraitement);
	glGetShaderiv(CleDuShaderDeSommet, GL_INFO_LOG_LENGTH, &InfoLogLongueur);
	if ( InfoLogLongueur > 0 )
	{
		std::vector<char> MessageDErreur(InfoLogLongueur+1);
		glGetShaderInfoLog(CleDuShaderDeSommet, InfoLogLongueur, NULL, &MessageDErreur[0]);
		std::cout << &MessageDErreur[0] << std::endl;
	}

	// COMPILATION DU SHADER DE FRAGMENT
	std::cout << "Compilation du shader de fragment : " << fragment_file_path << std::endl;
	{
		char const * PointeurChaine = CodeSourceDuShaderDeFragment.c_str();
		glShaderSource(CleDuShaderDeFragment, 1, &PointeurChaine , NULL);
	}
	glCompileShader(CleDuShaderDeFragment);

	// VERIFICATION DE LA COMPILATION DU SHADER DE FRAGMENT
	glGetShaderiv(CleDuShaderDeFragment, GL_COMPILE_STATUS, &ResultatTraitement);
	glGetShaderiv(CleDuShaderDeFragment, GL_INFO_LOG_LENGTH, &InfoLogLongueur);
	if ( InfoLogLongueur > 0 )
	{
		std::vector<char> MessageDErreur(InfoLogLongueur+1);
		glGetShaderInfoLog(CleDuShaderDeFragment, InfoLogLongueur, NULL, &MessageDErreur[0]);
		std::cout << &MessageDErreur[0] << std::endl;
	}

	// EDITION DE LIEN DES SHADERS
	std::cout << "Edition de lien entre les shaders ..." << std::endl;
	GLuint CleDuProgramme = glCreateProgram();
	glAttachShader(CleDuProgramme, CleDuShaderDeSommet);
	glAttachShader(CleDuProgramme, CleDuShaderDeFragment);
	glLinkProgram(CleDuProgramme);

	// VERIFICATION DE L'EDITION DE LIEN
	glGetProgramiv(CleDuProgramme, GL_LINK_STATUS, &ResultatTraitement);
	glGetProgramiv(CleDuProgramme, GL_INFO_LOG_LENGTH, &InfoLogLongueur);
	if ( InfoLogLongueur > 0 )
	{
		std::vector<char> MessageDErreur(InfoLogLongueur+1);
		glGetProgramInfoLog(CleDuProgramme, InfoLogLongueur, NULL, &MessageDErreur[0]);
		std::cout << &MessageDErreur[0] << std::endl;
	}

	// ON PEUT SUPPRIMER LES REFERENCES AUX SHADERS DE SOMMET ET DE FRAGMENT
	glDeleteShader(CleDuShaderDeSommet);
	glDeleteShader(CleDuShaderDeFragment);

	return CleDuProgramme;
}
