#include <iostream> // pour utiliser std::cout std::endl << >>

#include <GL/glew.h>
#include <GLFW/glfw3.h>

#include <glm/glm.hpp>
#include <glm/gtc/type_ptr.hpp>

#include "main.h"
#include "LisEtCompileLesShaders.h"


#define TAILLE_HORIZONTALE 513
#define TAILLE_VERTICALE 513

GLFWwindow* MaFenetre;
GLuint CleDeMaCouleurFinale;
GLuint DiagonaleVAO;
GLuint PointVAO;
GLuint MonProgrammeGLSL;

int main( int argc, char *argv[])
{
	// INITIALISATION DE LA FENETRE ET DU CONTEXTE OPENGL
	GLFWwindow *MaFenetre = InitialiseUnContexteEtUneFenetreOpenGL();


	// INITIALISATION DU PROGRAMME DE RENDU GLSL
	MonProgrammeGLSL = LisEtCompileLesShaders( "../shader/TraitementDesSommets.glsl", "../shader/TraitementDesFragments.glsl");
	glUseProgram(MonProgrammeGLSL);

	std::cout << "MonProgrammeGLSL = " << MonProgrammeGLSL << std::endl;

	// ON VA CHERCHER DANS LE PROGRAMME DE RENDU LA REFERENCE VERS LA VARIABLE CONTENANT LA COULEUR
	GLuint CleDeMaCouleurFinale = glGetUniformLocation( MonProgrammeGLSL, "MaCouleurFinale");
	std::cout << "CleDeMaCouleurFinale = " << CleDeMaCouleurFinale << std::endl;

	// INITIALISATION DES GEOMETRIES: DIAGONALE ET POINT

	GLuint DiagonaleVBO = TransfereLaDiagonaleSurLaCarteGraphique();
	std::cout << "DiagonaleVBO = " << DiagonaleVBO << std::endl;
	DiagonaleVAO = EncapsuleLaGeometrieDansUnVao( DiagonaleVBO);
	std::cout<<"DiagonaleVAO = "<< DiagonaleVAO<< std::endl;

	GLuint PointVBO = TransfereLePointSurLaCarteGraphique();
	std::cout << "PointVBO = " << PointVBO << std::endl;
	PointVAO = EncapsuleLaGeometrieDansUnVao( PointVBO);
	std::cout<<"PointVAO = "<< PointVAO<< std::endl;

	while (!glfwWindowShouldClose( MaFenetre)) {

		TraiteLesEvenementsClavier( MaFenetre);

		Rendu();

// EN CHOISISSANT "PollEvents" LE REDESSIN EST SYSTEMATIQUE
// EN CHOISISSANT "WaitEvents" LE REDESSIN N A LIEU QUE SI UN EVENEMENT EST EMIS
//		glfwPollEvents();
		glfwWaitEvents();
	}

	glDeleteBuffers(1, &PointVBO);
	glDeleteVertexArrays(1, &PointVAO);
	glDeleteVertexArrays(1, &DiagonaleVAO);

	glfwTerminate();
}

void Rendu()
{
	static int nRendu = 0;

	std::cout << "je lance le rendu #"<< nRendu++ << "                            \r";
	std::cout.flush();

	glfwMakeContextCurrent(MaFenetre);

	glClear( GL_COLOR_BUFFER_BIT );

	glUseProgram(MonProgrammeGLSL);

	glBindVertexArray( PointVAO);
	glUniform4fv( CleDeMaCouleurFinale, 1, glm::value_ptr(glm::vec4(0.,0.,1.,1.)));
	glDrawArrays( GL_POINTS, 0, 1);
	glBindVertexArray( 0);

	glUniform4fv( CleDeMaCouleurFinale, 1, glm::value_ptr(glm::vec4(0.,0.,0.,1.)));
	glBindVertexArray( DiagonaleVAO);
	glDrawArrays( GL_LINES, 0, 2);
	glBindVertexArray( 0);

	glfwSwapBuffers(MaFenetre);
}

GLuint TransfereLaDiagonaleSurLaCarteGraphique()
{
	glm::vec3 MesSommets[2];

	typedef enum {DiagonaleParLesCentres, 
			DiagonaleParLesCoinsHautGauche,
			DiagonaleParLesCoinsBasDroit,
			DiagonaleParLAreteEnBasPartieDroite,
			DiagonaleParLAreteEnBasPartieGauche,
			DiagonaleParLAreteAGauchePartieBasse,
			DiagonaleParLAreteAGauchePartieHaute,
	 } CalageDiagonale;

	CalageDiagonale OuEstLaDiagonale = DiagonaleParLAreteEnBasPartieGauche;

	switch (OuEstLaDiagonale) {
		case DiagonaleParLesCentres:
			glfwSetWindowTitle( MaFenetre, "Diagonale passant par les centres des pixels");
			MesSommets[0]=glm::vec3(0., 0., 0.);
			MesSommets[1]=glm::vec3(513., 513., 0.);
		break;
		case DiagonaleParLesCoinsHautGauche:
			glfwSetWindowTitle( MaFenetre, "Diagonale passant par les coins en haut a gauche des pixels");
			MesSommets[0]=glm::vec3(0., 1., 0.);
			MesSommets[1]=glm::vec3(512., 513., 0.);
		break;
		case DiagonaleParLesCoinsBasDroit:
			glfwSetWindowTitle( MaFenetre, "Diagonale passant par les coins en bas a droite des pixels");
			MesSommets[0]=glm::vec3(1., 0., 0.);
			MesSommets[1]=glm::vec3(513., 512., 0.);
		break;
		case DiagonaleParLAreteEnBasPartieDroite:
			glfwSetWindowTitle( MaFenetre, "Diagonale passant par la partie de droite de l arete du bas des pixels");
			MesSommets[0]=glm::vec3(0.7, 0., 0.);
			MesSommets[1]=glm::vec3(513., 512.3, 0.);
		break;
		case DiagonaleParLAreteEnBasPartieGauche:
			glfwSetWindowTitle( MaFenetre, "Diagonale passant par la partie gauche de l arete du bas des pixels");
			MesSommets[0]=glm::vec3(0.3, 0., 0.);
			MesSommets[1]=glm::vec3(513., 512.7, 0.);
		break;
		case DiagonaleParLAreteAGauchePartieBasse:
			glfwSetWindowTitle( MaFenetre, "Diagonale passant par la partie basse de l arete gauche des pixels");
			MesSommets[0]=glm::vec3(0., 0.3, 0.),
			MesSommets[1]=glm::vec3(512.7, 513., 0.);
		break;
		case DiagonaleParLAreteAGauchePartieHaute:
			glfwSetWindowTitle( MaFenetre, "Diagonale passant par la partie haute de l arete gauche des pixels");
			MesSommets[0]=glm::vec3(0., 0.7, 0.);
			MesSommets[1]=glm::vec3(512.3, 513., 0.);
		break;
	}

	GLuint vboID;
	glGenBuffers( 1, &vboID);
	glBindBuffer(GL_ARRAY_BUFFER, vboID);
	glBufferData(GL_ARRAY_BUFFER, sizeof(MesSommets), MesSommets, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	return vboID;
}
	
GLuint TransfereLePointSurLaCarteGraphique()
{
	glm::vec3 MonSommet[1] = {glm::vec3( 255.5, 255.5, 0.)};

	GLuint vboID;
	glGenBuffers( 1, &vboID);
	glBindBuffer(GL_ARRAY_BUFFER, vboID);
	glBufferData(GL_ARRAY_BUFFER, sizeof(MonSommet), MonSommet, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	return vboID;
}

GLuint EncapsuleLaGeometrieDansUnVao(GLuint MaGeometrie)
{
	GLuint vaoID;

	glGenVertexArrays( 1, &vaoID);
	glBindVertexArray(vaoID);

	glEnableVertexAttribArray(0); // ID de l'attribut à configurer

        glBindBuffer(GL_ARRAY_BUFFER, MaGeometrie); // ON ACTIVE LE VBO

       // On lui indique comment lire les donnees
	glVertexAttribPointer(
            0,                  // indice de l'attribut à configurer, correspond à la ligne "layout(location=0) in vec3 MonSommetEnEntree;" dans le vertex shader
            3,                  // nombre de composante par position (x, y, z)
            GL_FLOAT,           // type des composantes
            GL_FALSE,           // normalisation des composantes
            0,                  // decalage des composantes
            (void*)0            // offset des composantes
        );
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindVertexArray(0);

	return vaoID;
}

GLFWwindow* InitialiseUnContexteEtUneFenetreOpenGL()
{
	
	// Initialisation de GLFW
	if( !glfwInit() )
	{
		std::cout << "Echec de l'initialisation de GLFW" << std::endl;
		exit(EXIT_FAILURE);
	}

//	glfwWindowHint(GLFW_SAMPLES, 4); // Anti Aliasing
	glfwWindowHint(GLFW_SAMPLES, 0); // Anti Aliasing: TRES IMPORTANT POUR CETTE ILLUSTRATION DE RASTERISATION DE SUPPRIMER TOUT ANTIALIASING
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3); // OpenGL 4.1
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);


	// Ouverture d'une fenêtre en 512x512
	// et creation d'un contexte OpenGL
	if( !(MaFenetre = glfwCreateWindow(TAILLE_HORIZONTALE, TAILLE_VERTICALE, "Polytech_INFO4", NULL,NULL)) )
	{
		std::cout << "Echec de l'ouverture de fenetre OpenGL" << std::endl;
		glfwTerminate();
		exit(EXIT_FAILURE);
	}

	// PROCEDURE APPELLEE QUAND LA FENETRE CHANGE DE TAILLE
	glfwSetFramebufferSizeCallback( MaFenetre, UneFenetreChangeDeTaille);

    	// Autorise GLFW a recevoir les appuis de touche
	glfwSetInputMode( MaFenetre, GLFW_STICKY_KEYS, GL_TRUE);

    // Initialisation de  GLEW
	glfwMakeContextCurrent(MaFenetre);
	glewExperimental = GL_TRUE; 
	if (glewInit() != GLEW_OK) {
		std::cout << "Echec de l'initialisation de GLEW" << std::endl;
		exit(EXIT_FAILURE);
	}

	glLineWidth( 0.5); // EPAISSEUR DES LIGNES

	glClearColor( 1., 1., 1., 0.);
	glEnable(GL_VERTEX_PROGRAM_POINT_SIZE);


	// VOIR http://www.glfw.org/docs/latest/quick.html#quick_render POUR LES EXPLICATIONS SUR LA LIGNE SUIVANTE
	glfwSwapInterval(1);

	// Affichage des donnees du contexte OpenGL
	const GLubyte* renderer = glGetString (GL_RENDERER);
	std::cout << "Carte Graphique : " << renderer << std::endl;

	const GLubyte* version = glGetString (GL_VERSION);
	std::cout << "Driver OpenGL : " << version << std::endl;

	const GLubyte* shading_version = glGetString (GL_SHADING_LANGUAGE_VERSION);
	std::cout << "Shading Language: " << shading_version << std::endl;

	return MaFenetre;
}

void UneFenetreChangeDeTaille( GLFWwindow* UneFenetre, int Largeur, int Hauteur)
{
	std::cout << "Nouvelle taille de fenetre: "<< Largeur << " x " << Hauteur << std::endl;

	Rendu();
}

void TraiteLesEvenementsClavier(GLFWwindow *MaFenetre)
{
	// A TITRE D'EXEMPLE ON INDIQUE COMMENT DETECTER SI LA TOUCHE 'A' EST PRESSEE LORSQUE LA FENETRE EST ACTIVE
	if (glfwGetKey( MaFenetre, GLFW_KEY_A ) == GLFW_PRESS) {
		std::cout << "Touche A\n";
	}
}


